@@ -62,13 +62,18 @@ class Review(Base):
 
         self.facts.append(fact)
 
-    def add_violation(self, key, value, points):
+    def add_violation(self, key, value, points, domain):
         if self.is_complete:
             raise ValueError("Can't add anything to a completed review.")
 
         from holmes.models.violation import Violation  # to avoid circular dependency
 
-        violation = Violation(key=key, value=value, points=int(float(points)))
+        violation = Violation(
+            key=key,
+            value=value,
+            points=int(float(points)),
+            domain=domain
+        )
 
         self.violations.append(violation)
 
@@ -96,26 +101,100 @@ class Review(Base):
         return len(self.violations)
 
     @classmethod
-    def get_by_violation_key_name(cls, db, key_id, current_page=1, page_size=10):
-
-        from holmes.models.violation import Violation  # to avoid circular dependency
+    def _filter_violation_key_name(cls, db, query, key_id, domain_filter=None, page_filter=None):
+        from holmes.models.domain import Domain  # to avoid circular dependency
         from holmes.models.page import Page  # to avoid circular dependency
+        from holmes.models.violation import Violation  # to avoid circular dependency
+
+        query = query \
+            .filter(Page.id == Review.page_id) \
+            .filter(Violation.review_id == Review.id) \
+            .filter(Review.is_active == 1) \
+            .filter(Violation.key_id == key_id)
+
+        page_filter_prefix = '%'
+        if domain_filter:
+            domain = Domain.get_domain_by_name(domain_filter, db)
+            if domain:
+                query = query.filter(Review.domain_id == domain.id)
+                page_filter_prefix = domain.url
+
+        if page_filter:
+            query = query.filter(
+                Page.url.like(
+                    '{0}{1}%'.format(page_filter_prefix, page_filter)
+                )
+            )
+
+        return query
+
+    @classmethod
+    def count_by_violation_key_name(cls, db, key_id, domain_filter=None, page_filter=None):
+        from holmes.models.review import Review  # to avoid circular dependency
+        from holmes.models.violation import Violation  # to avoid circular dependency
+
+        query = db \
+            .query(sa.func.count(Violation.id)) \
+            .filter(Violation.review_is_active == 1) \
+            .filter(Violation.key_id == key_id)
+
+        page_filter_prefix = '%'
+        if domain_filter:
+            from holmes.models.domain import Domain  # to avoid circular dependency
+            domain = Domain.get_domain_by_name(domain_filter, db)
+            if domain:
+                query = query.filter(Review.domain_id == domain.id)
+                page_filter_prefix = domain.url
+
+        if page_filter:
+            from holmes.models.page import Page  # to avoid circular dependency
+            query = query.filter(Review.id == Violation.review_id) \
+                .filter(Page.id == Review.page_id) \
+                .filter(
+                    Page.url.like(
+                        '{0}{1}%'.format(page_filter_prefix, page_filter)
+                    )
+                )
+
+        return query.scalar()
+
+    @classmethod
+    def get_by_violation_key_name(cls, db, key_id, current_page=1, page_size=10, domain_filter=None, page_filter=None):
+        from holmes.models.page import Page  # to avoid circular dependency
+        from holmes.models.review import Review  # to avoid circular dependency
+        from holmes.models.violation import Violation  # to avoid circular dependency
 
         lower_bound = (current_page - 1) * page_size
         upper_bound = lower_bound + page_size
 
-        return db \
+        query = db \
             .query(
                 Review.uuid.label('review_uuid'),
                 Page.url,
                 Page.uuid.label('page_uuid'),
                 Review.completed_date
             ) \
-            .filter(Page.id == Review.page_id) \
-            .filter(Violation.review_id == Review.id) \
-            .filter(Review.is_active == True) \
             .filter(Violation.key_id == key_id) \
-            .order_by(Review.completed_date.desc())[lower_bound:upper_bound]
+            .filter(Review.id == Violation.review_id) \
+            .filter(Review.is_active == 1) \
+            .filter(Page.id == Review.page_id) \
+
+        page_filter_prefix = '%'
+        if domain_filter:
+            from holmes.models.domain import Domain  # to avoid circular dependency
+            domain = Domain.get_domain_by_name(domain_filter, db)
+            if domain:
+                query = query.filter(Review.domain_id == domain.id)
+                page_filter_prefix = domain.url
+
+        if page_filter:
+            query = query.filter(
+                Page.url.like(
+                    '{0}{1}%'.format(page_filter_prefix, page_filter)
+                )
+            )
+
+        return query.order_by(Review.completed_date.desc())[lower_bound:upper_bound]
 
     @classmethod
     def save_review(cls, page_uuid, review_data, db, fact_definitions, violation_definitions, cache, publish):
@@ -143,7 +222,12 @@ class Review(Base):
         for violation in review_data['violations']:
             name = violation['key']
             key = violation_definitions[name]['key']
-            review.add_violation(key, violation['value'], violation['points'])
+            review.add_violation(
+                key,
+                violation['value'],
+                violation['points'],
+                page.domain
+            )
 
         for i in range(3):
             db.begin(subtransactions=True)
@@ -174,6 +258,9 @@ class Review(Base):
                 page.domain,
                 increment=page.violations_count
             )
+
+            cache.increment_next_jobs_count(-1)
+
         else:
             old_violations_count = len(last_review.violations)
             new_violations_count = len(review.violations)
@@ -186,6 +273,8 @@ class Review(Base):
             for i in range(3):
                 db.begin(subtransactions=True)
                 try:
+                    for violation in last_review.violations:
+                        violation.review_is_active = False
                     last_review.is_active = False
                     db.commit()
                     break
@@ -197,6 +286,8 @@ class Review(Base):
                         db.rollback()
                         raise
 
+
+
         publish(dumps({
             'type': 'new-review',
             'reviewId': str(review.uuid)
